<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use App\Models\Order;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;

/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class ReturnSaleController extends Controller
{
    protected $title = 'Sales Returns (Refunds)';
    protected $viewPath = 'sales/returns';
    protected $link = 'sales/returns';
    protected $orderStatus = ['0' => 'Pending', '3' => 'Processing', '4' => 'Completed', '9' => 'On Hold', '5' => 'Cancelled'];

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->sorts = [
            'abandoned_orders.created_at-desc' => 'Latest Added',
            'abandoned_orders.created_at-asc' => 'Oldest Added',
            'billing_first_name-desc' => 'Customer Name Desc',
            'billing_first_name-asc' => 'Customer Name Asc',
            'billing_city-desc' => 'Location Desc',
            'billing_city-asc' => 'Location Asc',
            'amount_to_pay-desc' => 'Revenue Desc',
            'amount_to_pay-asc' => 'Revenue Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath, compact('page', 'request'));
    }

    public function lists(Request $request)
    {
        $sort = ['created_at', 'desc'];
        $detail = Order::select('id', 'order_no', 'invoice_no', 'created_at', 'billing_first_name', 'billing_last_name', 'billing_city', 'billing_postcode', 'paid_amount', 'order_status')
            ->paid()->online();
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('billing_first_name', 'like', "%{$names}%")
                            ->orWhere('billing_last_name', 'like', "%{$names}%")
                            ->orWhere('billing_city', 'like', "%{$names}%")
                            ->orWhere('billing_postcode', 'like', "%{$names}%")
                            ->orWhere('invoice_no', 'like', "%{$names}%")
                            ->orWhere('order_no', 'like', "%{$names}%");
                    });
                }
                if ($search['value'] != NULL && $search['name'] == 'status') {
                    $detail->where('order_status', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('created_at', '>=', $from_date)
                    ->whereDate('created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('created_at', '=', $to_date);
            }
        }

       
        $detail->where('order_status', 5)->groupBy('id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->editColumn('billing_first_name', function ($detail) {
                return $detail->billing_first_name . ' ' . $detail->billing_last_name;
            })
            ->editColumn('billing_postcode', function ($detail) {
                return $detail->billing_city . ', ' . strtoupper(str_replace(' ', '', $detail->billing_postcode));
            })
            ->editColumn('paid_amount', function ($detail) {
                return '&pound; ' . round($detail->paid_amount, 2);
            })
            ->editColumn('created_at', function ($detail) {
                return Carbon::parse($detail->created_at)->format('d/m/Y h:i a');
            })->editColumn('order_status', function ($detail) {
                return $this->orderStatus[$detail->order_status];
            })
            ->addColumn('action', function ($detail) {
                $product_url = url('sales/' . $detail->id);
                return HtmlHelper::showButton($product_url, 'Order Detail', 'link');
            })
            ->removeColumn('id', 'billing_last_name', 'billing_city')
            ->escapeColumns([])
            ->make(false);
    }



    public function show($id)
    {
        $order = Order::findOrFail($id);
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->order_status = $this->orderStatus;
        return view($this->viewPath . '/detail', compact('page', 'order'));
    }

     public function salesGraph(Request $request)
    {
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = 5;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? 'all';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
       
        return Order::getBySaleDay($criteria);
    }

    public function revenueGraph(Request $request)
    {
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = 5;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
        return Order::getByRevenueDay($criteria);
    }

    public function salesData(Request $request)
    {
        
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;
        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
        
        $order = number_format(Order::getAll($criteria)->where('order_status', 5)->count());
        
        $revenue = number_format(Order::getAll($criteria)->where('order_status', 5)->sum('paid_amount'), 2);

       
        return compact('order', 'revenue');
    }
    public function salesDataDashboard(Request $request)
    {
        $filter = $request->filter;
        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $abandon = Order::join('orders as o', 'o.id', 'abandoned_orders.order_id')
            ->when($filter != '' && $filter != 'all', function ($query) use ($filter, $start_date, $end_date) {
                $query->when($filter == 'today', function ($q) use ($end_date) {
                    $q->whereDate('abandoned_orders.created_at', '=', $end_date);
                    return $q;
                })
                    ->when($filter != 'today', function ($q) use ($start_date, $end_date) {
                        $q->whereDate('abandoned_orders.created_at', '>=', $start_date)
                            ->whereDate('abandoned_orders.created_at', '<=', $end_date);
                        return $q;
                    });
            })
            ->where('o.payment_status', 1);
        $abandon_count = number_format($abandon->count());
        $abandons = $abandon->select('abandoned_orders.order_id', 'amount_to_pay', 'abandoned_orders.created_at', 'billing_first_name', 'billing_last_name')
            ->orderBy('created_at', 'desc')->take(5)->get();
        return compact('abandon_count', 'abandons');
    }
}
