<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use App\Models\Order;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;

/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class SaleController extends Controller
{
    protected $title = 'Sales & Revenue';
    protected $viewPath = 'sales';
    protected $link = 'sales';
    protected $orderStatus = ['0' => 'Pending', '3' => 'Processing', '4' => 'Completed', '9' => 'On Hold', '5' => 'Cancelled'];

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->sorts = [
            'created_at-desc' => 'Latest Order',
            'created_at-asc' => 'Oldest Order',
            'invoice_no-desc' => 'Invoice No Desc',
            'invoice_no-asc' => 'Invoice No Asc',
            'billing_first_name-desc' => 'Customer Name Desc',
            'billing_first_name-asc' => 'Customer Name Asc',
            'billing_city-desc' => 'Location Desc',
            'billing_city-asc' => 'Location Asc',
            'paid_amount-desc' => 'Revenue Desc',
            'paid_amount-asc' => 'Revenue Asc',
        ];
        $page->order_status = $this->orderStatus;
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/lists', compact('page', 'request'));
    }

    public function lists(Request $request)
    {
        $sort = ['created_at', 'desc'];
        $detail = Order::select('id', 'order_no', 'invoice_no', 'created_at', 'billing_first_name', 'billing_last_name', 'billing_city', 'billing_postcode', 'paid_amount', 'order_status')
            ->paid()->online();
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('billing_first_name', 'like', "%{$names}%")
                            ->orWhere('billing_last_name', 'like', "%{$names}%")
                            ->orWhere('billing_city', 'like', "%{$names}%")
                            ->orWhere('billing_postcode', 'like', "%{$names}%")
                            ->orWhere('invoice_no', 'like', "%{$names}%")
                            ->orWhere('order_no', 'like', "%{$names}%");
                    });
                }
                if ($search['value'] != NULL && $search['name'] == 'status') {
                    $detail->where('order_status', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('created_at', '>=', $from_date)
                    ->whereDate('created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('created_at', '=', $to_date);
            }
        }
        $detail->groupBy('id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->editColumn('billing_first_name', function ($detail) {
                return $detail->billing_first_name . ' ' . $detail->billing_last_name;
            })
            ->editColumn('billing_postcode', function ($detail) {
                return $detail->billing_city . ', ' . strtoupper(str_replace(' ', '', $detail->billing_postcode));
            })
            ->editColumn('paid_amount', function ($detail) {
                return '&pound; ' . round($detail->paid_amount, 2);
            })
            ->editColumn('created_at', function ($detail) {
                return Carbon::parse($detail->created_at)->format('d/m/Y h:i a');
            })
            ->editColumn('order_status', function ($detail) {
                return $this->orderStatus[$detail->order_status];
            })
            ->addColumn('action', function ($detail) {
                $product_url = url('sales/' . $detail->id);
                return HtmlHelper::showButton($product_url, 'Order Detail', 'link');
            })
            ->removeColumn('id', 'billing_last_name', 'billing_city')
            ->escapeColumns([])
            ->make(false);
    }

    public function show($id)
    {
        $order = Order::findOrFail($id);
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->order_status = $this->orderStatus;
        return view($this->viewPath . '/detail', compact('page', 'order'));
    }

    public function salesGraph(Request $request)
    {
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? 'all';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
        return Order::getBySaleDay($criteria);
    }

    public function revenueGraph(Request $request)
    {
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
        return Order::getByRevenueDay($criteria);
    }

    public function salesData(Request $request)
    {
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;
        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
        $order = number_format(Order::getAll($criteria)->count());
        $revenue = number_format(Order::getAll($criteria)->sum('paid_amount'), 2);
        return compact('order', 'revenue');
    }

    public function salesDataDashboard(Request $request)
    {
        $filter = $request->filter;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $today = Carbon::now();
        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = '';
        $counts = [];
        $counts['order'] = number_format(Order::getAll($criteria)->count());
        $counts['revenue'] = number_format(Order::getAll($criteria)->sum('paid_amount'), 2);
        $counts['next_days'] = number_format(Order::getAll($criteria)->where('created_at', $today)->whereIn('shipping_method', [2, 15, 17])->count());
        $counts['same_days'] = number_format(Order::getAll($criteria)->where('created_at', $today)->whereIn('shipping_method', [14, 16])->count());
        $counts['cancelled'] = number_format(Order::getAll($criteria)->where('order_status', 5)->count());

        $highest_orders = Order::getAll($criteria)
            ->select('invoice_no', 'paid_amount', 'id', 'created_at')
            ->where('order_status', '!=', 5)
            ->where('paid_amount', '>=', 50)
            ->orderBy('paid_amount', 'desc')
            ->take(5)->get();
        if (count($highest_orders) > 0) {
            foreach ($highest_orders as $order) {
                $order->added_on = Carbon::parse($order->created_at)->format('d/m/Y h:i A');
            }
        }
        return compact('counts', 'highest_orders');
    }
}
