<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use App\Models\Staff;
use App\Models\Order;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;

/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class StaffController extends Controller
{
    protected $title = 'Staff';
    protected $viewPath = 'staffs';
    protected $link = 'staffs';
    protected $orderStatus = ['0' => 'Pending', '1' => 'Active' ];

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->sorts = [
            'created_at-desc' => 'Latest Joined',
            'created_at-asc' => 'Oldest Joined',
            'user_id-desc' => 'No Desc',
            'user_id-asc' => 'No Asc',
            'first_name-desc' => 'Doctor Name Desc',
            'first_name-asc' => 'Doctor Name Asc',
            'profession-desc' => 'Profession Desc',
            'profession-asc' => 'Profession Asc',
        ];
        

        $page->order_status = $this->orderStatus;
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/lists', compact('page', 'request'));
    }

    public function lists(Request $request)
    {
        $sort = ['created_at', 'desc'];
        $detail =   Staff::select( 'id','first_name','last_name',
            'email','phone', 'created_at','status');


 
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
           
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('first_name', 'like', "%{$names}%")
                            ->orWhere('last_name', 'like', "%{$names}%")
                            ->orWhere('email', 'like', "%{$names}%")
                            ->orWhere('phone', 'like', "%{$names}%");


                    });
                }
                if ($search['value'] != NULL && $search['name'] == 'status') {
                    $detail->where('status', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
         
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('created_at', '>=', $from_date)
                    ->whereDate('created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('created_at', '=', $to_date);
            }
        }
        
        $detail->groupBy('id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->editColumn('first_name', function ($detail) {
                return $detail->first_name . ' ' . $detail->last_name;
            })
            ->editColumn('email', function ($detail) {
                return   $detail->email;
            })
            ->editColumn('phone', function ($detail) {
                return   $detail->phone;
            })
         
            ->editColumn('created_at', function ($detail) {
                return Carbon::parse($detail->created_at)->format('d/m/Y h:i a');
            })
            ->editColumn('status', function ($detail) {
                return $this->orderStatus[$detail->status];
            })->addColumn('action', function ($detail) {
                $action = '';
           
                $action .= HtmlHelper::editOnClickButton($detail->id, 'Staff Edit', 'cube',);
                return $action;
            })
            ->removeColumn('id','last_name')
            ->escapeColumns([])
            ->make(false);
    }





    public function store(Request $request)
    {
        $request->validate([
            'first_name'          => 'required',
            'last_name'          => 'required',
            'email'         => 'required|email',
            'mobile'        => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10',
        ]);


        $staff = new Staff();
        $staff->first_name = $request->first_name;
        $staff->last_name = $request->last_name;
        $staff->email = $request->email;
        $staff->phone = $request->mobile;
        $staff->save();

        return response()->json(['success'=>'Successfully']);
    }
 
    public function update(Request $request,$id)
    {
        $request->validate([
            'first_name'          => 'required',
            'last_name'          => 'required',
            'email'         => 'required|email',
            'mobile'        => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10',
        ]);


        $staff =  Staff::find($id);
        $staff->first_name = $request->first_name;
        $staff->last_name = $request->last_name;
        $staff->email = $request->email;
        $staff->phone = $request->mobile;
        $staff->save();

        return response()->json(['success'=>'Successfully']);
    }
 
}
