<?php

namespace App\Models;

use Carbon\Carbon;
use GuzzleHttp\Client;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Order extends Model
{
    public function scopePaid($query)
    {
        return $query->where('orders.payment_status', 0);
    }

    public function scopeOnline($query)
    {
        return $query->where('orders.order_from', 1);
    }

    public function scopeActive($query)
    {
        return $query->where('orders.order_status', '!=', 5);
    }

    public static function getAll($criteria)
    {
        $keyword = $criteria->keyword;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;
        $status = $criteria->status;
        return self::paid()->online()
            ->when($keyword, function ($query) use ($keyword) {
                return $query->where(function ($q) use ($keyword) {
                    $q->where('billing_first_name', 'like', "%{$keyword}%")
                        ->orWhere('billing_last_name', 'like', "%{$keyword}%")
                        ->orWhere('billing_city', 'like', "%{$keyword}%")
                        ->orWhere('billing_postcode', 'like', "%{$keyword}%")
                        ->orWhere('invoice_no', 'like', "%{$keyword}%")
                        ->orWhere('order_no', 'like', "%{$keyword}%");
                });
            })
            ->when($start_date == '' && $end_date != '', function ($q) use ($end_date) {
                $q->whereDate('created_at', '=', $end_date);
                return $q;
            })
            ->when($start_date != '' && $end_date == '', function ($q) use ($start_date) {
                $q->whereDate('created_at', '=', $start_date);
                return $q;
            })
            ->when($start_date != '' && $end_date != '', function ($q) use ($start_date, $end_date) {
                $q->whereDate('created_at', '>=', $start_date)
                    ->whereDate('created_at', '<=', $end_date);
                return $q;
            })
            ->when($status, function ($query) use ($status) {
                return $query->where('order_status', $status);
            });
    }

    public static function getProducts($criteria)
    {
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;
        $result = [];
        $products = self::select('p.title', DB::raw('sum(d.quantity) as qty'))
            ->join('order_details as d', 'orders.id', 'd.order_id')
            ->join('products as p', 'd.product_id', 'p.id')
            ->paid()->online()->active()
            ->when($filter != '' && $filter != 'all', function ($query) use ($filter, $start_date, $end_date) {
                $query->when($filter == 'today', function ($q) use ($end_date) {
                    $q->whereDate('orders.created_at', '=', $end_date);
                    return $q;
                })
                    ->when($filter != 'today', function ($q) use ($start_date, $end_date) {
                        $q->whereDate('orders.created_at', '<=', $start_date)
                            ->whereDate('orders.created_at', '>=', $end_date);
                        return $q;
                    });
            })
            ->groupBy('d.product_id')
            ->orderBy('qty', 'desc')
            ->limit(5)
            ->get();
        foreach ($products as $product) {
            $result[] = [$product->title, $product->qty];
        }
        return json_encode(['result' => $result]);
    }

    public static function getCategories($criteria)
    {
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;

        $categories = self::select('c.title', DB::raw('sum(d.quantity) as qty'))
            ->join('order_details as d', 'orders.id', 'd.order_id')
            ->join('category_product as cp', 'cp.product_id', 'd.product_id')
            ->join('product_categories as c', 'cp.category_id', 'c.id')
            ->paid()->online()->active()
            ->when($filter != '' && $filter != 'all', function ($query) use ($filter, $start_date, $end_date) {
                $query->when($filter == 'today', function ($q) use ($end_date) {
                    $q->whereDate('orders.created_at', '=', $end_date);
                    return $q;
                })
                    ->when($filter != 'today', function ($q) use ($start_date, $end_date) {
                        $q->whereDate('orders.created_at', '<=', $start_date)
                            ->whereDate('orders.created_at', '>=', $end_date);
                        return $q;
                    });
            })
            ->groupBy('cp.category_id')
            ->orderBy('qty', 'desc')
            ->limit(5)
            ->get();
        foreach ($categories as $category) {
            $result[] = [$category->title, $category->qty];
        }
        return json_encode(['result' => $result]);
    }

    public static function getCoupons($criteria)
    {
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;

        $coupons = self::select('coupon', DB::raw('count(id) as counts'))
            ->paid()->online()->active()
            ->whereNotNull('coupon')
            ->when($filter != '' && $filter != 'all', function ($query) use ($filter, $start_date, $end_date) {
                $query->when($filter == 'today', function ($q) use ($end_date) {
                    $q->whereDate('orders.created_at', '=', $end_date);
                    return $q;
                })
                    ->when($filter != 'today', function ($q) use ($start_date, $end_date) {
                        $q->whereDate('orders.created_at', '<=', $start_date)
                            ->whereDate('orders.created_at', '>=', $end_date);
                        return $q;
                    });
            })
            ->groupBy('coupon')
            ->orderBy('counts', 'desc')
            ->limit(5)
            ->get();
        foreach ($coupons as $coupon) {
            $result[] = $coupon->counts;
            $format[] = $coupon->coupon;
        }
        return json_encode(['format' => $format, 'result' => $result]);
    }

    public static function getBySales($filter)
    {
        $date = [];
        $result = [];
        $format = [];

        $check_date = Carbon::now()->format('Y-m-d');
        $date[] = $check_date;
        if ($filter == '') {
            for ($i = 1; $i <= 14; $i++) {
                $check_date = Carbon::now()->subDays($i)->format('Y-m-d');
                $date[] = $check_date;
            }
        } elseif ($filter == 'day') {
            $check_date = Carbon::now()->format('Y-m-d');
            $date[] = $check_date;
        } elseif ($filter == 'week') {
            $last_date = Carbon::now()->startOfWeek();
            for ($i = 1; $i <= 6; $i++) {
                $check_date = Carbon::now()->subDays($i)->format('Y-m-d');
                if ($last_date < $check_date) {
                    $date[] = $check_date;
                }
            }
        } elseif ($filter == 'month') {
            $last_date = Carbon::now()->startOfMonth();
            for ($i = 1; $i <= 29; $i += 2) {
                $check_date = Carbon::now()->subDays($i)->format('Y-m-d');
                if ($last_date < $check_date) {
                    $date[] = $check_date;
                }
            }
        } elseif ($filter == 'year') {
            $last_date = Carbon::now()->startOfYear();
            for ($i = 1; $i <= 11; $i++) {
                $check_date = Carbon::now()->subMonth($i)->format('Y-m-d');
                if ($last_date < $check_date) {
                    $date[] = $check_date;
                }
            }
        } elseif ($filter == 'lweek') {
            for ($i = 1; $i <= 6; $i++) {
                $check_date = Carbon::now()->subDays($i)->format('Y-m-d');
                $date[] = $check_date;
            }
        } elseif ($filter == 'lmonth') {
            for ($i = 1; $i <= 29; $i += 2) {
                $check_date = Carbon::now()->subDays($i)->format('Y-m-d');
                $date[] = $check_date;
            }
        } elseif ($filter == 'lyear') {
            for ($i = 1; $i <= 11; $i++) {
                $check_date = Carbon::now()->subMonth($i)->format('Y-m-d');
                $date[] = $check_date;
            }
        }
        $date = array_reverse($date);
        foreach ($date as $row) {
            if ($filter == 'year' || $filter == 'lyear') {
                $result[] = self::paid()->online()
                    ->whereMonth('created_at', Carbon::parse($row)->format('m'))
                    ->whereYear('created_at', Carbon::parse($row)->format('Y'))
                    ->count();
                $format[] = Carbon::parse($row)->format('M Y');
            } else {
                $result[] = self::paid()->online()
                    ->whereDate('created_at', $row)
                    ->count();
                $format[] = Carbon::parse($row)->format('d M');
            }
        }
        return json_encode(['date' => $format, 'result' => $result]);
    }

    public static function getByLocations($criteria)
    {
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;

        $results = [];
        $locations = self::select(DB::raw('TRIM(shipping_postcode) as postcode'), DB::raw('count(id) as count'))
            ->paid()->online()->active()
            ->whereNotNull('shipping_postcode')
            ->when($filter != '' && $filter != 'all', function ($query) use ($filter, $start_date, $end_date) {
                $query->when($filter == 'today', function ($q) use ($end_date) {
                    $q->whereDate('orders.created_at', '=', $end_date);
                    return $q;
                })
                    ->when($filter != 'today', function ($q) use ($start_date, $end_date) {
                        $q->whereDate('orders.created_at', '>=', $start_date)
                            ->whereDate('orders.created_at', '<=', $end_date);
                        return $q;
                    });
            })
            ->groupBy('postcode')
            ->get();
        
        if (count($locations) > 0) {
            foreach ($locations as $location) {
                $address = strtoupper(str_replace(' ', '', $location->postcode));
                dump($address);
                // if ($address != '') {
                //     $area = DB::table('postcode_geocodes')
                //         ->where('postcode', $address)
                //         ->first();
                //     if ($area) {
                //         if ($area->longitude != '') {
                //             $results[] = [
                //                 'longitude' => $area->longitude,
                //                 'latitude' => $area->latitude,
                //                 'text' => $area->city . ', ' . $area->postcode . ' - ' . $location->count . ' Orders',
                //             ];
                //         }
                //     } else {
                //         $client = new Client(); //GuzzleHttp\Client
                //         $url = 'https://maps.googleapis.com/maps/api/geocode/json?address=' . $address . '&key=' . config('services.google.key');
                //         $result = (string)$client->get($url)->getBody();
                //         $json = json_decode($result);
                //         if ($json->results) {
                //             $lat = $json->results[0]->geometry->location->lat;
                //             $lng = $json->results[0]->geometry->location->lng;
                //             $city = $json->results[0]->address_components[2]->short_name ?? $json->results[0]->address_components[1]->long_name;
                //             $data = [
                //                 'postcode' => $address,
                //                 'city' => $city,
                //                 'latitude' => $lat,
                //                 'longitude' => $lng,
                //             ];
                //             DB::table('postcode_geocodes')->insert($data);
                //             $results[] = [
                //                 'longitude' => $lng,
                //                 'latitude' => $lat,
                //                 'text' => $city . ', ' . $address . ' - ' . $location->count . ' Orders',
                //             ];
                //         } else {
                //             $data = [
                //                 'postcode' => $address,
                //                 'city' => '',
                //                 'latitude' => '',
                //                 'longitude' => '',
                //             ];
                //             DB::table('postcode_geocodes')->insert($data);
                //         }
                //     }
                // }
            }
        }
        dd('dsfsdfsd');
        return json_encode($results);
    }

    public static function getBySaleDay($criteria)
    {
        $keyword = $criteria->keyword;
        $filter = $criteria->type;
        $status = $criteria->status;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;

        $date = [];
        $result = [];
        $format = [];

        if ($start_date != '' && $end_date == '') {
            $date[] = $start_date;
        } elseif ($start_date == '' && $end_date != '') {
            $date[] = $end_date;
        } else {
            $start_date = $start_date == '' ? Carbon::now()->subDay(15) : $start_date;
            $end_date = $end_date == '' ? Carbon::now()->addDay() : $end_date->addDay();

            $formated_date = new \DatePeriod(
                new \DateTime($start_date),
                new \DateInterval('P1D'),
                new \DateTime($end_date)
            );
            foreach ($formated_date as $fd) {
                $date[] = $fd;
            }
            $date = array_reverse($date);
        }

        foreach ($date as $row) {
            $result[] = self::paid()->online()
                ->when($keyword, function ($query) use ($keyword) {
                    return $query->where(function ($q) use ($keyword) {
                        $q->where('billing_first_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_last_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_city', 'like', "%{$keyword}%")
                            ->orWhere('billing_postcode', 'like', "%{$keyword}%")
                            ->orWhere('invoice_no', 'like', "%{$keyword}%")
                            ->orWhere('order_no', 'like', "%{$keyword}%");
                    });
                })
                ->when($status, function ($query) use ($status) {
                    return $query->where('order_status', $status);
                })
                ->whereDate('created_at', '=', $row)
                ->count();
            $format[] = Carbon::parse($row)->format('d M');
        }

        return json_encode(['date' => $format, 'result' => $result]);
    }

    public static function getByRevenueDay($criteria)
    {
        $keyword = $criteria->keyword;
        $filter = $criteria->type;
        $status = $criteria->status;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $date = [];
        $result = [];
        $format = [];

        if ($start_date != '' && $end_date == '') {
            $date[] = $start_date;
        } elseif ($start_date == '' && $end_date != '') {
            $date[] = $end_date;
        } else {
            $start_date = $start_date == '' ? Carbon::now()->subDay(15) : $start_date;
            $end_date = $end_date == '' ? Carbon::now()->addDay() : $end_date->addDay();

            $formated_date = new \DatePeriod(
                new \DateTime($start_date),
                new \DateInterval('P1D'),
                new \DateTime($end_date)
            );
            foreach ($formated_date as $fd) {
                $date[] = $fd;
            }
            $date = array_reverse($date);
        }
        foreach ($date as $row) {
            $result[] = self::paid()->online()
                ->when($keyword, function ($query) use ($keyword) {
                    return $query->where(function ($q) use ($keyword) {
                        $q->where('billing_first_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_last_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_city', 'like', "%{$keyword}%")
                            ->orWhere('billing_postcode', 'like', "%{$keyword}%")
                            ->orWhere('invoice_no', 'like', "%{$keyword}%")
                            ->orWhere('order_no', 'like', "%{$keyword}%");
                    });
                })
                ->when($status, function ($query) use ($status) {
                    return $query->where('order_status', $status);
                })
                ->whereDate('created_at', '=', $row)
                ->sum('paid_amount');
            $format[] = Carbon::parse($row)->format('d M');
        }
        return json_encode(['date' => $format, 'result' => $result]);
    }

    public function customer()
    {
        return $this->belongsToMany(Customer::class, 'order_customers');
    }

    public function details()
    {
        return $this->hasMany(OrderDetail::class);
    }

    public function billingCountry()
    {
        return $this->belongsTo(Country::class, 'billing_country_id');
    }

    public function shippingCountry()
    {
        return $this->belongsTo(Country::class, 'shipping_country_id');
    }
}
